/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "build.h"

bool continue_build;
bool run_command(Glib::ustring, Gtk::TreeView*, EnvironmentSettings&);
Glib::ustring get_configure_parameters();

// Run the autogen.sh command if a project is currently open.
void OpenLDev::on_autogen()
{  
  if (proj.get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    notebook->set_current_page(0);
    chdir (proj.get_location().c_str());
    
    Glib::ustring command = string(ptemplate.get_autogen()) + " 2>&1";
    run_command(command, build_box, env);
  }
}

// Compile current file if a file and a project is open. The function then
// creates the compile string from the project language & runs the command
void OpenLDev::on_compile() 
{
  if (!noteman->get_file_open())
    error("Please open or create\na file first!");
  else 
  {
    notebook->set_current_page(0);
    Glib::ustring command = ptemplate.get_compile(noteman->get_current_filename());
    Glib::ustring location = Glib::ustring(proj.get_location()) + "src/";
    chdir (proj.get_location().c_str());

    command += " 2>&1";
    run_command(command, build_box, env);
  }
}

// If a project is open, build it. Run autogen.sh and then build again if
// Makefiles are not found. All of the files are first saved.
void OpenLDev::on_build() 
{
  if (proj.get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    notebook->set_current_page(0);
    noteman->save_all_files();
    chdir (proj.get_location().c_str());
    
    Glib::ustring command = string(ptemplate.get_build()) + " 2>&1";
    if (!run_command(command, build_box, env))
    {
      on_autogen();
      run_command(command, build_box, env);
    }
  }
}

// If a project is open, run the executable in the terminal specified
// by the user 
void OpenLDev::on_execute() 
{
  if (proj.get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    Glib::ustring dir = Glib::ustring(proj.get_location()) + "src/";
    chdir (dir.c_str());
    Glib::ustring command = Glib::ustring(env.get_terminal()) + " -e " 
      + Glib::ustring(proj.get_location()) + "src/" + Glib::ustring(proj.get_name());
    
    static FILE *exec_file;
    exec_file = popen(command.c_str(), "r");
  }
}

// If a project is open, run the ./configure command in the project directory
void OpenLDev::on_configure() 
{
  if (proj.get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    Glib::ustring parameters = get_configure_parameters();
    if (parameters != "CANCEL")
    {
      notebook->set_current_page(0);
      chdir (proj.get_location().c_str());
    
      Glib::ustring command = string(ptemplate.get_configure()) + " " + parameters + " 2>&1";
      run_command(command, build_box, env);
    }
  }
}

// Clean all of the sources, build files, object files and those files
// created by configure
void OpenLDev::on_clean()
{
  if (proj.get_name().empty())
    error("Please open or create\na project first!");
  else 
  {
    notebook->set_current_page(0);
    chdir (proj.get_location().c_str());
    
    Glib::ustring command = string(ptemplate.get_clean()) + " 2>&1";
    run_command(command, build_box, env);
  }
}

// Stop the current compilation, clean, configure, autogen or build
void OpenLDev::on_stop() 
{ continue_build = false;
}

// Run the specified command with pipes. This function receives the command to
// run, the TreeView object and the environment settings class object. It 
// cleans the build output and runs the command.
//
// This function has the ability to highlight warnings, errors and other compiler
// messages with the user-specified colors as well as check if the build is able
// to find Makefiles
bool run_command(Glib::ustring command, Gtk::TreeView *build_box, EnvironmentSettings &env)
{
  char buff[32];
  FILE *in;
  bool successful = true, autogenerate = false;
  int location;
  Glib::ustring text = "\0", line;
  BuildColumns build_columns;
  Glib::RefPtr<Gtk::ListStore> build_model;
  Gtk::CellRendererText *renderer;
  Gtk::TreeModel::Row row;
  Gtk::TreeViewColumn *color_column;
  continue_build = true;
  
  // Clean up build_box & get it ready to accept output
  build_box->remove_all_columns();
  build_model = Gtk::ListStore::create(build_columns);
  build_box->set_model(build_model);
  
  // Append the columns & create the cell renderer
  build_box->append_column("Compiler Output", build_columns.col_text);
  color_column = build_box->get_column(0);
  Gtk::CellRenderer *cell_renderer = color_column->get_first_cell_renderer();
  renderer = dynamic_cast<Gtk::CellRendererText*>(cell_renderer);
  color_column->add_attribute(renderer->property_foreground(), build_columns.col_color);
  
  // Add the command to the box
  row = *(build_model->append());
  row[build_columns.col_text] = command.substr(0, command.find("2>&1", 0));
  row[build_columns.col_color] = env.get_warning_color();
  build_box->get_selection()->select(build_box->get_model()->get_iter("0"));
    
  // Open pipe & loop until the program is complete
  in = popen(command.c_str(), "r");
  
  while (fgets(buff, sizeof(buff), in) && continue_build) 
  {
    text += buff;
    // Find the endl's & split them to be added to build_box
    location = text.find("\n",0);
    while (location != Glib::ustring::npos) 
    {
      // Check to see if autogen.sh must be run before build
      if (text.find("No targets specified",0) != string::npos)
        autogenerate = true;
        
      // Append line & erase it from the buffer
      line = text.substr(0,location);
      row = *(build_model->append());
      row[build_columns.col_text] = line;
      text.erase(0,location+1);
      location = text.find("\n",0);

      // Apply Error or Warning colors
      if (line.find("error: ") != Glib::ustring::npos) 
      {
        row[build_columns.col_color] = env.get_error_color();
        successful = false;
      } 
      else if (line.find(": ") != Glib::ustring::npos)
        row[build_columns.col_color] = env.get_warning_color();
      else
        row[build_columns.col_color] = "#000000";
      
      // Update the build_box when there is output
      while(Gtk::Main::instance()->events_pending()) 
      {
        Gtk::Main::instance()->iteration();
        build_box->scroll_to_row(build_model->get_path(row));
      }
    }
  }
    
  // Close the stream, & print whether command was successful or not
  pclose(in);
  row = *(build_model->append());
  if (successful)
    row[build_columns.col_text] = "Completed ... Successful";
  else
    row[build_columns.col_text] = "Completed ... Failed";
  row[build_columns.col_color] = "#000000";
  build_box->scroll_to_row(build_model->get_path(row));
  
  // Return false if the user must run autogen.sh before building & then do it
  if (autogenerate)
  {
    row = *(build_model->append());
    row[build_columns.col_text] = "You must run 'Auto Generate' first!";
    row[build_columns.col_color] = env.get_warning_color();
    return false;
  }
  return true;
}

Glib::ustring get_configure_parameters()
{
  Glib::ustring return_text;
  Gtk::Dialog *configure_dialog = new Gtk::Dialog();
  Gtk::Button *cancel = new Gtk::Button(Gtk::StockID("gtk-cancel"));
  Gtk::Button *ok = new Gtk::Button(Gtk::StockID("gtk-ok"));
  Gtk::Label *label = new Gtk::Label("Parameters:");
  Gtk::Entry *parameters = new Gtk::Entry();
  Gtk::HBox *hbox = new Gtk::HBox(false, 10);
  
  ok->set_flags(Gtk::CAN_DEFAULT);
  hbox->set_spacing(10);
  hbox->pack_start(*label, Gtk::PACK_SHRINK, 0);
  hbox->pack_start(*parameters);
  
  configure_dialog->get_vbox()->pack_start(*hbox);
  configure_dialog->set_border_width(5);
  configure_dialog->set_size_request(350, 100);
  configure_dialog->set_title("Configure Parameters");
  configure_dialog->set_modal(true);
  configure_dialog->set_resizable(false);
  configure_dialog->property_destroy_with_parent().set_value(true);
  configure_dialog->set_has_separator(true);
  configure_dialog->add_action_widget(*cancel, -6);
  configure_dialog->add_action_widget(*ok, -5);
  configure_dialog->show_all_children();

  switch (configure_dialog->run())
  {
    case (Gtk::RESPONSE_OK):
      return_text = parameters->get_text();
      break;
    default:
      return_text = "CANCEL";
      break;
  }
  
  configure_dialog->hide();
  return return_text;
}
